import React, { useEffect, useLayoutEffect, useRef, RefObject, CSSProperties } from 'react';
import classnames from 'classnames';
import { useFloating, autoUpdate, offset, size, detectOverflow, hide } from '@floating-ui/react-dom';
import type { Coords } from '@floating-ui/react-dom';

import styles from './frame.module.scss';

/** frame padding around the target */
const MARGIN_FROM_TARGET = +styles.offset || 6; // setting fallback 6, for tests
/** min. distance from the edge of the screen. */
const MARGIN_FROM_DOC_EDGE = 0;

const overflowParameters = { rootBoundary: 'document', padding: MARGIN_FROM_DOC_EDGE } as const;
const SHIFT_POSITIVE = 'shiftPositive';

const HAS_RESIZE_OBSERVER = typeof window !== 'undefined' && !!window.ResizeObserver;

export interface FrameProps extends React.HTMLAttributes<HTMLDivElement> {
  /** apply the frame to this element  */
  targetRef: RefObject<HTMLElement | null>;
  /**
   * the specific flavor of the frame.
   * @default "redBorderClass"
   */
  stylesClass?: string;
  /** continually update frame position to match moving elements */
  watchMotion?: boolean;
}

// position - bottom start (bottom left corner)
// x - width - horizontal (cross axis)
// y - height - vertical (main axis)

type DimensionsStyle = Pick<CSSProperties, 'width' | 'height' | 'maxWidth' | 'maxHeight'>;

export function Frame({ targetRef, watchMotion, className, stylesClass = styles.overlayBorder, style }: FrameProps) {
  const dimensionRef = useRef<DimensionsStyle>({ width: 0, height: 0 });

  const { x, y, strategy, reference, floating, update, refs, middlewareData } = useFloating({
    placement: 'bottom-start',
    middleware: [
      // replace dimensions from previous iterations with the target's size
      // this is only the measured size, not yet the applied size
      {
        name: 'align-to-target',
        fn({ rects }) {
          rects.floating = {
            ...rects.floating,
            width: rects.reference.width + 2 * MARGIN_FROM_TARGET,
            height: rects.reference.height + 2 * MARGIN_FROM_TARGET,
          };

          return {};
        },
      },
      // reposition x,y, to the top of the reference
      offset(({ rects }) => -rects.reference.height),
      // offset the frame by its extra padding
      offset(() => ({ mainAxis: -MARGIN_FROM_TARGET, crossAxis: -MARGIN_FROM_TARGET })),
      // pushes the frame into the document. Similar to shift(), but only pushes when coods are negative
      {
        name: 'shiftPositive',
        fn: async (args) => {
          const overflow = await detectOverflow(args, overflowParameters);

          const nextCoords = {
            x: overflow.left > 0 ? args.x + overflow.left : args.x,
            y: overflow.top > 0 ? args.y + overflow.top : args.y,
          };
          const shiftAmount = {
            x: nextCoords.x - args.x,
            y: nextCoords.y - args.y,
          };

          return {
            ...nextCoords,
            data: shiftAmount,
          };
        },
      },
      // size also applies overflow detection via width and height
      size({
        // apply overflow detection in reference to the document
        rootBoundary: 'document',
        padding: MARGIN_FROM_DOC_EDGE,
        apply({ elements, rects, availableHeight, availableWidth, middlewareData }) {
          const shift: Coords = middlewareData[SHIFT_POSITIVE] || { x: 0, y: 0 };
          const paddingX = 2 * MARGIN_FROM_TARGET - shift.x;
          const paddingY = 2 * MARGIN_FROM_TARGET - shift.y;

          const dimensions: DimensionsStyle = {
            width: rects.reference.width + paddingX,
            height: rects.reference.height + paddingY,
            maxWidth: availableWidth,
            maxHeight: availableHeight,
          };

          // per floating-ui docs, apply styles directly during apply()
          Object.assign(elements.floating.style, dimensions);

          // also store in reference, so react renders will have the same value
          dimensionRef.current = dimensions;
        },
      }),
      hide({ strategy: 'referenceHidden' }),
    ],
  });

  // set target as floating reference
  useLayoutEffect(() => {
    reference(targetRef.current);
  }, [targetRef.current]);

  // automatically update on scroll, resize, etc.
  // `watchMotion` will trigger continuous updates using animation frame
  useEffect(() => {
    if (!refs.reference.current || !refs.floating.current || !HAS_RESIZE_OBSERVER) return () => {};

    return autoUpdate(refs.reference.current, refs.floating.current, update, { animationFrame: watchMotion });
  }, [refs.reference.current, refs.floating.current, update, watchMotion]);

  // could check if x !== null
  const isReady = !middlewareData.hide?.referenceHidden;

  return (
    <div
      ref={floating}
      className={classnames(className, stylesClass, !isReady && styles.hidden)}
      style={{
        ...style,
        ...dimensionRef.current,
        position: strategy,
        // starting at pos [0,0] will ensure the label doesn't increase the document size.
        top: y ?? 0,
        left: x ?? 0,
      }}
    />
  );
}
